@@ -47,6 +47,9 @@ struct sim_data {
 };
 
 static const char *crsm_prefix[] = { "+CRSM:", NULL };
+static const char *cpin_prefix[] = { "+CPIN:", NULL };
+static const char *clck_prefix[] = { "+CLCK:", NULL };
+static const char *none_prefix[] = { NULL };
 
 static void at_crsm_info_cb(gboolean ok, GAtResult *result, gpointer user_data)
 {
@@ -60,7 +63,6 @@ static void at_crsm_info_cb(gboolean ok, GAtResult *result, gpointer user_data)
 	int str;
 	unsigned char access[3];
 
-	dump_response("at_crsm_info_cb", ok, result);
 	decode_at_error(&error, g_at_result_final_response(result));
 
 	if (!ok) {
@@ -81,7 +83,7 @@ static void at_crsm_info_cb(gboolean ok, GAtResult *result, gpointer user_data)
 			(sw1 == 0x90 && sw2 != 0x00))
 		goto error;
 
-	ofono_debug("crsm_info_cb: %02x, %02x, %i", sw1, sw2, len);
+	DBG("crsm_info_cb: %02x, %02x, %i", sw1, sw2, len);
 
 	if (response[0] == 0x62)
 		ok = sim_parse_3g_get_response(response, len, &flen, &rlen,
@@ -137,7 +139,6 @@ static void at_crsm_read_cb(gboolean ok, GAtResult *result,
 	const guint8 *response;
 	gint sw1, sw2, len;
 
-	dump_response("at_crsm_read_cb", ok, result);
 	decode_at_error(&error, g_at_result_final_response(result));
 
 	if (!ok) {
@@ -162,7 +163,7 @@ static void at_crsm_read_cb(gboolean ok, GAtResult *result,
 		return;
 	}
 
-	ofono_debug("crsm_read_cb: %02x, %02x, %d", sw1, sw2, len);
+	DBG("crsm_read_cb: %02x, %02x, %d", sw1, sw2, len);
 
 	cb(&error, response, len, cbd->data);
 }
@@ -226,7 +227,6 @@ static void at_crsm_update_cb(gboolean ok, GAtResult *result,
 	struct ofono_error error;
 	gint sw1, sw2;
 
-	dump_response("at_crsm_update_cb", ok, result);
 	decode_at_error(&error, g_at_result_final_response(result));
 
 	if (!ok) {
@@ -250,7 +250,7 @@ static void at_crsm_update_cb(gboolean ok, GAtResult *result,
 		return;
 	}
 
-	ofono_debug("crsm_update_cb: %02x, %02x", sw1, sw2);
+	DBG("crsm_update_cb: %02x, %02x", sw1, sw2);
 
 	cb(&error, cbd->data);
 }
@@ -364,7 +364,6 @@ static void at_cimi_cb(gboolean ok, GAtResult *result, gpointer user_data)
 	const char *imsi;
 	int i;
 
-	dump_response("at_cimi_cb", ok, result);
 	decode_at_error(&error, g_at_result_final_response(result));
 
 	if (!ok) {
@@ -379,7 +378,7 @@ static void at_cimi_cb(gboolean ok, GAtResult *result, gpointer user_data)
 
 	imsi = g_at_result_iter_raw_line(&iter);
 
-	ofono_debug("cimi_cb: %s", imsi);
+	DBG("cimi_cb: %s", imsi);
 
 	cb(&error, imsi, cbd->data);
 }
@@ -429,6 +428,7 @@ static struct {
 static void at_cpin_cb(gboolean ok, GAtResult *result, gpointer user_data)
 {
 	struct cb_data *cbd = user_data;
+	struct sim_data *sd = ofono_sim_get_data(cbd->user);
 	GAtResultIter iter;
 	ofono_sim_passwd_cb_t cb = cbd->cb;
 	struct ofono_error error;
@@ -436,24 +436,32 @@ static void at_cpin_cb(gboolean ok, GAtResult *result, gpointer user_data)
 	int pin_type = OFONO_SIM_PASSWORD_INVALID;
 	int i;
 	int len = sizeof(at_sim_name) / sizeof(*at_sim_name);
+	const char *final = g_at_result_final_response(result);
 
-	dump_response("at_cpin_cb", ok, result);
-	decode_at_error(&error, g_at_result_final_response(result));
+	if (sd->vendor == OFONO_VENDOR_WAVECOM && ok && strlen(final) > 7)
+		decode_at_error(&error, "OK");
+	else
+		decode_at_error(&error, final);
 
 	if (!ok) {
 		cb(&error, -1, cbd->data);
 		return;
 	}
 
-	g_at_result_iter_init(&iter, result);
+	if (sd->vendor == OFONO_VENDOR_WAVECOM) {
+		/* +CPIN: <pin> */
+		pin_required = final + 7;
+	} else {
+		g_at_result_iter_init(&iter, result);
 
-	if (!g_at_result_iter_next(&iter, "+CPIN:")) {
-		CALLBACK_WITH_FAILURE(cb, -1, cbd->data);
-		return;
+		if (!g_at_result_iter_next(&iter, "+CPIN:")) {
+			CALLBACK_WITH_FAILURE(cb, -1, cbd->data);
+			return;
+		}
+
+		g_at_result_iter_next_unquoted_string(&iter, &pin_required);
 	}
 
-	g_at_result_iter_next_unquoted_string(&iter, &pin_required);
-
 	for (i = 0; i < len; i++) {
 		if (strcmp(pin_required, at_sim_name[i].name))
 			continue;
@@ -467,7 +475,7 @@ static void at_cpin_cb(gboolean ok, GAtResult *result, gpointer user_data)
 		return;
 	}
 
-	ofono_debug("crsm_pin_cb: %s", pin_required);
+	DBG("crsm_pin_cb: %s", pin_required);
 
 	cb(&error, pin_type, cbd->data);
 }
@@ -481,7 +489,9 @@ static void at_pin_query(struct ofono_sim *sim, ofono_sim_passwd_cb_t cb,
 	if (!cbd)
 		goto error;
 
-	if (g_at_chat_send(sd->chat, "AT+CPIN?", NULL,
+	cbd->user = sim;
+
+	if (g_at_chat_send(sd->chat, "AT+CPIN?", cpin_prefix,
 				at_cpin_cb, cbd, g_free) > 0)
 		return;
 
@@ -499,7 +509,6 @@ static void at_lock_unlock_cb(gboolean ok, GAtResult *result,
 	ofono_sim_lock_unlock_cb_t cb = cbd->cb;
 	struct ofono_error error;
 
-	dump_response("at_lock_unlock_cb", ok, result);
 	decode_at_error(&error, g_at_result_final_response(result));
 
 	cb(&error, cbd->data);
@@ -518,7 +527,7 @@ static void at_pin_send(struct ofono_sim *sim, const char *passwd,
 
 	snprintf(buf, sizeof(buf), "AT+CPIN=\"%s\"", passwd);
 
-	ret = g_at_chat_send(sd->chat, buf, NULL,
+	ret = g_at_chat_send(sd->chat, buf, none_prefix,
 				at_lock_unlock_cb, cbd, g_free);
 
 	memset(buf, 0, sizeof(buf));
@@ -547,7 +556,7 @@ static void at_pin_send_puk(struct ofono_sim *sim, const char *puk,
 
 	snprintf(buf, sizeof(buf), "AT+CPIN=\"%s\",\"%s\"", puk, passwd);
 
-	ret = g_at_chat_send(sd->chat, buf, NULL,
+	ret = g_at_chat_send(sd->chat, buf, none_prefix,
 				at_lock_unlock_cb, cbd, g_free);
 
 	memset(buf, 0, sizeof(buf));
@@ -593,7 +602,7 @@ static void at_pin_enable(struct ofono_sim *sim,
 	snprintf(buf, sizeof(buf), "AT+CLCK=\"%s\",%i,\"%s\"",
 			at_clck_cpwd_fac[passwd_type], enable ? 1 : 0, passwd);
 
-	ret = g_at_chat_send(sd->chat, buf, NULL,
+	ret = g_at_chat_send(sd->chat, buf, none_prefix,
 				at_lock_unlock_cb, cbd, g_free);
 
 	memset(buf, 0, sizeof(buf));
@@ -629,7 +638,7 @@ static void at_change_passwd(struct ofono_sim *sim,
 	snprintf(buf, sizeof(buf), "AT+CPWD=\"%s\",\"%s\",\"%s\"",
 			at_clck_cpwd_fac[passwd_type], old, new);
 
-	ret = g_at_chat_send(sd->chat, buf, NULL,
+	ret = g_at_chat_send(sd->chat, buf, none_prefix,
 				at_lock_unlock_cb, cbd, g_free);
 
 	memset(buf, 0, sizeof(buf));
@@ -653,7 +662,6 @@ static void at_lock_status_cb(gboolean ok, GAtResult *result,
 	struct ofono_error error;
 	int locked;
 
-	dump_response("at_lock_status_cb", ok, result);
 	decode_at_error(&error, g_at_result_final_response(result));
 
 	if (!ok) {
@@ -670,7 +678,7 @@ static void at_lock_status_cb(gboolean ok, GAtResult *result,
 
 	g_at_result_iter_next_number(&iter, &locked);
 
-	ofono_debug("lock_status_cb: %i", locked);
+	DBG("lock_status_cb: %i", locked);
 
 	cb(&error, locked, cbd->data);
 }
@@ -693,7 +701,7 @@ static void at_pin_query_enabled(struct ofono_sim *sim,
 	snprintf(buf, sizeof(buf), "AT+CLCK=\"%s\",2",
 			at_clck_cpwd_fac[passwd_type]);
 
-	if (g_at_chat_send(sd->chat, buf, NULL,
+	if (g_at_chat_send(sd->chat, buf, clck_prefix,
 				at_lock_status_cb, cbd, g_free) > 0)
 		return;
 
@@ -704,83 +712,6 @@ error:
 	CALLBACK_WITH_FAILURE(cb, -1, data);
 }
 
-static void at_csim_envelope_cb(gboolean ok, GAtResult *result,
-		gpointer user_data)
-{
-	struct cb_data *cbd = user_data;
-	GAtResultIter iter;
-	ofono_sim_read_cb_t cb = cbd->cb;
-	struct ofono_error error;
-	const guint8 *response;
-	gint rlen, len;
-
-	dump_response("at_csim_envelope_cb", ok, result);
-	decode_at_error(&error, g_at_result_final_response(result));
-
-	if (!ok)
-		goto error;
-
-	g_at_result_iter_init(&iter, result);
-
-	if (!g_at_result_iter_next(&iter, "+CSIM:"))
-		goto error;
-
-	if (!g_at_result_iter_next_number(&iter, &rlen))
-		goto error;
-
-	if (!g_at_result_iter_next_hexstring(&iter, &response, &len))
-		goto error;
-
-	if (rlen != len * 2 || len < 2 ||
-			response[len - 2] != 0x90 || response[len - 1] != 0)
-		goto error;
-
-	ofono_debug("csim_envelope_cb: %i", len);
-
-	cb(&error, response, len - 2, cbd->data);
-	return;
-
-error:
-	CALLBACK_WITH_FAILURE(cb, NULL, 0, cbd->data);
-}
-
-static void at_sim_envelope(struct ofono_sim *sim, int length,
-				const guint8 *command,
-				ofono_sim_read_cb_t cb, void *data)
-{
-	struct sim_data *sd = ofono_sim_get_data(sim);
-	struct cb_data *cbd = cb_data_new(cb, data);
-	char *buf = g_try_new(char, 64 + length * 2);
-	int len, ret;
-
-	if (!cbd || !buf)
-		goto error;
-
-	len = sprintf(buf, "AT+CSIM=%i,A0C20000%02hhX",
-			10 + length * 2, length);
-
-	for (; length; length--)
-		len += sprintf(buf + len, "%02hhX", *command++);
-
-	ret = g_at_chat_send(sd->chat, buf, crsm_prefix,
-				at_csim_envelope_cb, cbd, g_free);
-
-	g_free(buf);
-	buf = NULL;
-
-	if (ret > 0)
-		return;
-
-error:
-	if (buf)
-		g_free(buf);
-
-	if (cbd)
-		g_free(cbd);
-
-	CALLBACK_WITH_FAILURE(cb, NULL, 0, data);
-}
-
 static gboolean at_sim_register(gpointer user)
 {
 	struct ofono_sim *sim = user;
@@ -800,6 +731,9 @@ static int at_sim_probe(struct ofono_sim *sim, unsigned int vendor,
 	sd->chat = chat;
 	sd->vendor = vendor;
 
+	if (sd->vendor == OFONO_VENDOR_WAVECOM)
+		g_at_chat_add_terminator(chat, "+CPIN:", 6, TRUE);
+
 	ofono_sim_set_data(sim, sd);
 	g_idle_add(at_sim_register, sim);
 
@@ -833,7 +767,6 @@ static struct ofono_sim_driver driver = {
 	.lock			= at_pin_enable,
 	.change_passwd		= at_change_passwd,
 	.query_locked		= at_pin_query_enabled,
-	.envelope		= at_sim_envelope,
 };
 
 void at_sim_init()
