@@ -2,7 +2,7 @@
  *
  *  oFono - Open Telephony stack for Linux
  *
- *  Copyright (C) 2008-2009  Intel Corporation. All rights reserved.
+ *  Copyright (C) 2008-2010  Intel Corporation. All rights reserved.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License version 2 as
@@ -32,6 +32,21 @@ extern "C" {
 #define OFONO_MANAGER_INTERFACE "org.ofono.Manager"
 #define OFONO_MANAGER_PATH "/"
 #define OFONO_MODEM_INTERFACE "org.ofono.Modem"
+#define OFONO_CALL_BARRING_INTERFACE "org.ofono.CallBarring"
+#define OFONO_CALL_FORWARDING_INTERFACE "org.ofono.CallForwarding"
+#define OFONO_CALL_METER_INTERFACE "org.ofono.CallMeter"
+#define OFONO_CALL_SETTINGS_INTERFACE "org.ofono.CallSettings"
+#define OFONO_CALL_VOLUME_INTERFACE OFONO_SERVICE ".CallVolume"
+#define OFONO_CBS_MANAGER_INTERFACE "org.ofono.CbsManager"
+#define OFONO_MESSAGE_WAITING_INTERFACE "org.ofono.MessageWaiting"
+#define OFONO_NETWORK_REGISTRATION_INTERFACE "org.ofono.NetworkRegistration"
+#define OFONO_NETWORK_OPERATOR_INTERFACE "org.ofono.NetworkOperator"
+#define OFONO_PHONEBOOK_INTERFACE "org.ofono.Phonebook"
+#define OFONO_RADIO_SETTINGS_INTERFACE "org.ofono.RadioSettings"
+#define OFONO_SIM_MANAGER_INTERFACE "org.ofono.SimManager"
+#define OFONO_SMS_MANAGER_INTERFACE "org.ofono.SmsManager"
+#define OFONO_VOICECALL_INTERFACE "org.ofono.VoiceCall"
+#define OFONO_VOICECALL_MANAGER_INTERFACE "org.ofono.VoiceCallManager"
 
 /* Essentially a{sv} */
 #define OFONO_PROPERTIES_ARRAY_SIGNATURE DBUS_DICT_ENTRY_BEGIN_CHAR_AS_STRING \
@@ -47,6 +62,9 @@ void ofono_dbus_dict_append(DBusMessageIter *dict, const char *key, int type,
 void ofono_dbus_dict_append_array(DBusMessageIter *dict, const char *key,
 					int type, void *val);
 
+void ofono_dbus_dict_append_dict(DBusMessageIter *dict, const char *key,
+					int type, void *val);
+
 int ofono_dbus_signal_property_changed(DBusConnection *conn, const char *path,
 					const char *interface, const char *name,
 					int type, void *value);
@@ -57,6 +75,12 @@ int ofono_dbus_signal_array_property_changed(DBusConnection *conn,
 						const char *name, int type,
 						void *value);
 
+int ofono_dbus_signal_dict_property_changed(DBusConnection *conn,
+						const char *path,
+						const char *interface,
+						const char *name, int type,
+						void *value);
+
 #ifdef __cplusplus
 }
 #endif
