@@ -2,7 +2,7 @@
  *
  *  oFono - Open Source Telephony
  *
- *  Copyright (C) 2008-2009  Intel Corporation. All rights reserved.
+ *  Copyright (C) 2008-2010  Intel Corporation. All rights reserved.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License version 2 as
@@ -26,8 +26,6 @@
 extern "C" {
 #endif
 
-#include <stdbool.h>
-
 #include <ofono/types.h>
 
 struct ofono_modem;
@@ -43,13 +41,17 @@ const char *ofono_modem_get_path(struct ofono_modem *modem);
 void ofono_modem_set_data(struct ofono_modem *modem, void *data);
 void *ofono_modem_get_data(struct ofono_modem *modem);
 
-struct ofono_modem *ofono_modem_create(const char *type);
+struct ofono_modem *ofono_modem_create(const char *name, const char *type);
 int ofono_modem_register(struct ofono_modem *modem);
 void ofono_modem_remove(struct ofono_modem *modem);
 
 void ofono_modem_set_powered(struct ofono_modem *modem, ofono_bool_t powered);
 ofono_bool_t ofono_modem_get_powered(struct ofono_modem *modem);
 
+ofono_bool_t ofono_modem_get_online(struct ofono_modem *modem);
+
+void ofono_modem_set_name(struct ofono_modem *modem, const char *name);
+
 int ofono_modem_set_string(struct ofono_modem *modem,
 				const char *key, const char *value);
 const char *ofono_modem_get_string(struct ofono_modem *modem, const char *key);
@@ -59,8 +61,12 @@ int ofono_modem_set_integer(struct ofono_modem *modem,
 int ofono_modem_get_integer(struct ofono_modem *modem, const char *key);
 
 int ofono_modem_set_boolean(struct ofono_modem *modem,
-				const char *key, bool value);
-bool ofono_modem_get_boolean(struct ofono_modem *modem, const char *key);
+				const char *key, ofono_bool_t value);
+ofono_bool_t ofono_modem_get_boolean(struct ofono_modem *modem,
+					const char *key);
+
+typedef void (*ofono_modem_online_cb)(const struct ofono_error *error,
+					void *data);
 
 struct ofono_modem_driver {
 	const char *name;
@@ -78,11 +84,18 @@ struct ofono_modem_driver {
 	/* Power down device */
 	int (*disable)(struct ofono_modem *modem);
 
+	/* Enable or disable cellular radio */
+	void (*set_online)(struct ofono_modem *modem, ofono_bool_t online,
+				ofono_modem_online_cb callback, void *data);
+
 	/* Populate the atoms available without SIM / Locked SIM */
 	void (*pre_sim)(struct ofono_modem *modem);
 
 	/* Populate the atoms that are available with SIM / Unlocked SIM*/
 	void (*post_sim)(struct ofono_modem *modem);
+
+	/* Populate the atoms available online */
+	void (*post_online)(struct ofono_modem *modem);
 };
 
 int ofono_modem_driver_register(const struct ofono_modem_driver *);
