@@ -42,17 +42,24 @@ static int next_modem_id = 0;
 static gboolean powering_down = FALSE;
 static int modems_remaining = 0;
 
-enum ofono_property_type {
-	OFONO_PROPERTY_TYPE_INVALID = 0,
-	OFONO_PROPERTY_TYPE_STRING,
-	OFONO_PROPERTY_TYPE_INTEGER,
-	OFONO_PROPERTY_TYPE_BOOLEAN,
+enum property_type {
+	PROPERTY_TYPE_INVALID = 0,
+	PROPERTY_TYPE_STRING,
+	PROPERTY_TYPE_INTEGER,
+	PROPERTY_TYPE_BOOLEAN,
+};
+
+enum modem_state {
+	MODEM_STATE_POWER_OFF,
+	MODEM_STATE_PRE_SIM,
+	MODEM_STATE_OFFLINE,
+	MODEM_STATE_ONLINE,
 };
 
 struct ofono_modem {
 	char			*path;
+	enum modem_state	modem_state;
 	GSList			*atoms;
-	GSList			*pre_sim_atoms;
 	struct ofono_watchlist	*atom_watches;
 	GSList			*interface_list;
 	unsigned int		call_ids;
@@ -61,6 +68,7 @@ struct ofono_modem {
 	ofono_bool_t		powered;
 	ofono_bool_t		powered_pending;
 	guint			timeout;
+	ofono_bool_t		online;
 	GHashTable		*properties;
 	struct ofono_sim	*sim;
 	unsigned int		sim_watch;
@@ -83,19 +91,20 @@ struct ofono_devinfo {
 
 struct ofono_atom {
 	enum ofono_atom_type type;
+	enum modem_state modem_state;
 	void (*destruct)(struct ofono_atom *atom);
 	void (*unregister)(struct ofono_atom *atom);
 	void *data;
 	struct ofono_modem *modem;
 };
 
-struct ofono_atom_watch {
+struct atom_watch {
 	struct ofono_watchlist_item item;
 	enum ofono_atom_type type;
 };
 
-struct ofono_property {
-	enum ofono_property_type type;
+struct modem_property {
+	enum property_type type;
 	void *value;
 };
 
@@ -160,6 +169,7 @@ struct ofono_atom *__ofono_modem_add_atom(struct ofono_modem *modem,
 	atom = g_new0(struct ofono_atom, 1);
 
 	atom->type = type;
+	atom->modem_state = modem->modem_state;
 	atom->destruct = destruct;
 	atom->data = data;
 	atom->modem = modem;
@@ -190,7 +200,7 @@ static void call_watches(struct ofono_atom *atom,
 	struct ofono_modem *modem = atom->modem;
 	GSList *atom_watches = modem->atom_watches->items;
 	GSList *l;
-	struct ofono_atom_watch *watch;
+	struct atom_watch *watch;
 	ofono_atom_watch_func notify;
 
 	for (l = atom_watches; l; l = l->next) {
@@ -235,12 +245,12 @@ unsigned int __ofono_modem_add_atom_watch(struct ofono_modem *modem,
 					ofono_atom_watch_func notify,
 					void *data, ofono_destroy_func destroy)
 {
-	struct ofono_atom_watch *watch;
+	struct atom_watch *watch;
 
 	if (notify == NULL)
 		return 0;
 
-	watch = g_new0(struct ofono_atom_watch, 1);
+	watch = g_new0(struct atom_watch, 1);
 
 	watch->type = type;
 	watch->item.notify = notify;
@@ -257,14 +267,6 @@ gboolean __ofono_modem_remove_atom_watch(struct ofono_modem *modem,
 	return __ofono_watchlist_remove_item(modem->atom_watches, id);
 }
 
-#define FIND_ATOM_IN_LIST(list)			\
-	for (l = list; l; l = l->next) {	\
-		atom = l->data;			\
-						\
-		if (atom->type == type)		\
-			return atom;		\
-	}					\
-
 struct ofono_atom *__ofono_modem_find_atom(struct ofono_modem *modem,
 						enum ofono_atom_type type)
 {
@@ -274,22 +276,16 @@ struct ofono_atom *__ofono_modem_find_atom(struct ofono_modem *modem,
 	if (modem == NULL)
 		return NULL;
 
-	FIND_ATOM_IN_LIST(modem->atoms)
-	FIND_ATOM_IN_LIST(modem->pre_sim_atoms);
+	for (l = modem->atoms; l; l = l->next) {
+		atom = l->data;
+
+		if (atom->type == type)
+			return atom;
+	}
 
 	return NULL;
 }
 
-#define FOREACH_ATOM_IN_LIST(list)		\
-	for (l = list; l; l = l->next) {	\
-		atom = l->data;			\
-						\
-		if (atom->type != type)		\
-			continue;		\
-						\
-		callback(atom, data);		\
-	}					\
-
 void __ofono_modem_foreach_atom(struct ofono_modem *modem,
 				enum ofono_atom_type type,
 				ofono_atom_func callback, void *data)
@@ -300,8 +296,14 @@ void __ofono_modem_foreach_atom(struct ofono_modem *modem,
 	if (modem == NULL)
 		return;
 
-	FOREACH_ATOM_IN_LIST(modem->atoms)
-	FOREACH_ATOM_IN_LIST(modem->pre_sim_atoms)
+	for (l = modem->atoms; l; l = l->next) {
+		atom = l->data;
+
+		if (atom->type != type)
+			continue;
+
+		callback(atom, data);
+	}
 }
 
 void __ofono_atom_free(struct ofono_atom *atom)
@@ -309,7 +311,6 @@ void __ofono_atom_free(struct ofono_atom *atom)
 	struct ofono_modem *modem = atom->modem;
 
 	modem->atoms = g_slist_remove(modem->atoms, atom);
-	modem->pre_sim_atoms = g_slist_remove(modem->pre_sim_atoms, atom);
 
 	__ofono_atom_unregister(atom);
 
@@ -319,13 +320,23 @@ void __ofono_atom_free(struct ofono_atom *atom)
 	g_free(atom);
 }
 
-static void remove_all_atoms(GSList **atoms)
+static void flush_atoms(struct ofono_modem *modem, enum modem_state new_state)
 {
-	GSList *l;
-	struct ofono_atom *atom;
+	GSList *cur;
+	GSList *prev;
+	GSList *tmp;
 
-	for (l = *atoms; l; l = l->next) {
-		atom = l->data;
+	prev = NULL;
+	cur = modem->atoms;
+
+	while (cur) {
+		struct ofono_atom *atom = cur->data;
+
+		if (atom->modem_state <= new_state) {
+			prev = cur;
+			cur = cur->next;
+			continue;
+		}
 
 		__ofono_atom_unregister(atom);
 
@@ -333,10 +344,140 @@ static void remove_all_atoms(GSList **atoms)
 			atom->destruct(atom);
 
 		g_free(atom);
+
+		if (prev)
+			prev->next = cur->next;
+		else
+			modem->atoms = cur->next;
+
+		tmp = cur;
+		cur = cur->next;
+		g_slist_free_1(tmp);
+	}
+}
+
+static void modem_change_state(struct ofono_modem *modem,
+				enum modem_state new_state)
+{
+	struct ofono_modem_driver const *driver = modem->driver;
+	enum modem_state old_state = modem->modem_state;
+	ofono_bool_t new_online = new_state == MODEM_STATE_ONLINE;
+
+	if (old_state == new_state)
+		return;
+
+	if (new_online != modem->online) {
+		DBusConnection *conn = ofono_dbus_get_connection();
+		modem->online = new_online;
+		ofono_dbus_signal_property_changed(conn, modem->path,
+					OFONO_MODEM_INTERFACE, "Online",
+					DBUS_TYPE_BOOLEAN, &modem->online);
 	}
 
-	g_slist_free(*atoms);
-	*atoms = NULL;
+	modem->modem_state = new_state;
+
+	if (old_state > new_state)
+		flush_atoms(modem, new_state);
+
+	switch (new_state) {
+	case MODEM_STATE_POWER_OFF:
+		modem->call_ids = 0;
+		break;
+
+	case MODEM_STATE_PRE_SIM:
+		if (old_state < MODEM_STATE_PRE_SIM && driver->pre_sim)
+			driver->pre_sim(modem);
+		break;
+
+	case MODEM_STATE_OFFLINE:
+		if (old_state < MODEM_STATE_OFFLINE) {
+			if (driver->post_sim)
+				driver->post_sim(modem);
+			__ofono_history_probe_drivers(modem);
+			__ofono_nettime_probe_drivers(modem);
+		}
+		break;
+
+	case MODEM_STATE_ONLINE:
+		if (driver->post_online)
+			driver->post_online(modem);
+		break;
+	}
+}
+
+static void online_cb(const struct ofono_error *error, void *data)
+{
+	struct ofono_modem *modem = data;
+	DBusMessage *reply;
+
+	if (error->type == OFONO_ERROR_TYPE_NO_ERROR &&
+		modem->modem_state == MODEM_STATE_OFFLINE)
+		reply = dbus_message_new_method_return(modem->pending);
+	else
+		reply = __ofono_error_failed(modem->pending);
+
+	__ofono_dbus_pending_reply(&modem->pending, reply);
+
+	if (error->type == OFONO_ERROR_TYPE_NO_ERROR &&
+		modem->modem_state == MODEM_STATE_OFFLINE)
+		modem_change_state(modem, MODEM_STATE_ONLINE);
+}
+
+static void offline_cb(const struct ofono_error *error, void *data)
+{
+	struct ofono_modem *modem = data;
+	DBusMessage *reply;
+
+	if (error->type == OFONO_ERROR_TYPE_NO_ERROR)
+		reply = dbus_message_new_method_return(modem->pending);
+	else
+		reply = __ofono_error_failed(modem->pending);
+
+	__ofono_dbus_pending_reply(&modem->pending, reply);
+
+	if (error->type == OFONO_ERROR_TYPE_NO_ERROR &&
+		modem->modem_state == MODEM_STATE_ONLINE)
+		modem_change_state(modem, MODEM_STATE_OFFLINE);
+}
+
+static DBusMessage *set_property_online(struct ofono_modem *modem,
+					DBusMessage *msg,
+					DBusMessageIter *var)
+{
+	ofono_bool_t online;
+	const struct ofono_modem_driver *driver = modem->driver;
+
+	if (dbus_message_iter_get_arg_type(var) != DBUS_TYPE_BOOLEAN)
+		return __ofono_error_invalid_args(msg);
+
+	dbus_message_iter_get_basic(var, &online);
+
+	if (modem->pending != NULL)
+		return __ofono_error_busy(msg);
+
+	if (!driver->set_online)
+		return __ofono_error_not_implemented(msg);
+
+	if (modem->modem_state < MODEM_STATE_OFFLINE)
+		return __ofono_error_not_available(msg);
+
+	if (modem->online == online)
+		return dbus_message_new_method_return(msg);
+
+	modem->pending = dbus_message_ref(msg);
+
+	driver->set_online(modem, online,
+				online ? online_cb : offline_cb, modem);
+
+	return NULL;
+}
+
+ofono_bool_t ofono_modem_get_online(struct ofono_modem *modem)
+{
+	if (modem == NULL)
+		return FALSE;
+
+	return modem->online;
 }
 
 static DBusMessage *modem_get_properties(DBusConnection *conn,
@@ -361,6 +502,9 @@ static DBusMessage *modem_get_properties(DBusConnection *conn,
 					OFONO_PROPERTIES_ARRAY_SIGNATURE,
 					&dict);
 
+	ofono_dbus_dict_append(&dict, "Online", DBUS_TYPE_BOOLEAN,
+				&modem->online);
+
 	ofono_dbus_dict_append(&dict, "Powered", DBUS_TYPE_BOOLEAN,
 				&modem->powered);
 
@@ -420,11 +564,8 @@ static int set_powered(struct ofono_modem *modem, ofono_bool_t powered)
 		return -EALREADY;
 
 	/* Remove the atoms even if the driver is no longer available */
-	if (powered == FALSE) {
-		remove_all_atoms(&modem->atoms);
-		remove_all_atoms(&modem->pre_sim_atoms);
-		modem->call_ids = 0;
-	}
+	if (powered == FALSE)
+		modem_change_state(modem, MODEM_STATE_POWER_OFF);
 
 	modem->powered_pending = powered;
 
@@ -502,6 +643,9 @@ static DBusMessage *modem_set_property(DBusConnection *conn,
 
 	dbus_message_iter_recurse(&iter, &var);
 
+	if (g_str_equal(name, "Online"))
+		return set_property_online(modem, msg, &var);
+
 	if (g_str_equal(name, "Powered") == TRUE) {
 		ofono_bool_t powered;
 		int err;
@@ -535,10 +679,8 @@ static DBusMessage *modem_set_property(DBusConnection *conn,
 						"Powered", DBUS_TYPE_BOOLEAN,
 						&powered);
 
-		if (powered) {
-			if (modem->driver->pre_sim)
-				modem->driver->pre_sim(modem);
-		}
+		if (powered)
+			modem_change_state(modem, MODEM_STATE_PRE_SIM);
 
 		return NULL;
 	}
@@ -596,14 +738,10 @@ void ofono_modem_set_powered(struct ofono_modem *modem, ofono_bool_t powered)
 						"Powered", DBUS_TYPE_BOOLEAN,
 						&dbus_powered);
 
-		if (powered) {
-			if (modem->driver->pre_sim)
-				modem->driver->pre_sim(modem);
-		} else {
-			remove_all_atoms(&modem->atoms);
-			remove_all_atoms(&modem->pre_sim_atoms);
-			modem->call_ids = 0;
-		}
+		if (powered)
+			modem_change_state(modem, MODEM_STATE_PRE_SIM);
+		else
+			modem_change_state(modem, MODEM_STATE_POWER_OFF);
 	}
 
 	if (powering_down && powered == FALSE) {
@@ -924,7 +1062,7 @@ const char **__ofono_modem_get_list()
 
 static void unregister_property(gpointer data)
 {
-	struct ofono_property *property = data;
+	struct modem_property *property = data;
 
 	DBG("property %p", property);
 
@@ -933,32 +1071,31 @@ static void unregister_property(gpointer data)
 }
 
 static int set_modem_property(struct ofono_modem *modem, const char *name,
-				enum ofono_property_type type,
-				const void *value)
+				enum property_type type, const void *value)
 {
-	struct ofono_property *property;
+	struct modem_property *property;
 
 	DBG("modem %p property %s", modem, name);
 
-	if (type != OFONO_PROPERTY_TYPE_STRING &&
-			type != OFONO_PROPERTY_TYPE_INTEGER)
+	if (type != PROPERTY_TYPE_STRING &&
+			type != PROPERTY_TYPE_INTEGER)
 		return -EINVAL;
 
-	property = g_try_new0(struct ofono_property, 1);
+	property = g_try_new0(struct modem_property, 1);
 	if (property == NULL)
 		return -ENOMEM;
 
 	property->type = type;
 
 	switch (type) {
-	case OFONO_PROPERTY_TYPE_STRING:
+	case PROPERTY_TYPE_STRING:
 		property->value = g_strdup((const char *) value);
 		break;
-	case OFONO_PROPERTY_TYPE_INTEGER:
+	case PROPERTY_TYPE_INTEGER:
 		property->value = g_memdup(value, sizeof(int));
 		break;
-	case OFONO_PROPERTY_TYPE_BOOLEAN:
-		property->value = g_memdup(value, sizeof(bool));
+	case PROPERTY_TYPE_BOOLEAN:
+		property->value = g_memdup(value, sizeof(ofono_bool_t));
 		break;
 	default:
 		break;
@@ -970,10 +1107,10 @@ static int set_modem_property(struct ofono_modem *modem, const char *name,
 }
 
 static gboolean get_modem_property(struct ofono_modem *modem, const char *name,
-					enum ofono_property_type type,
+					enum property_type type,
 					void *value)
 {
-	struct ofono_property *property;
+	struct modem_property *property;
 
 	DBG("modem %p property %s", modem, name);
 
@@ -986,14 +1123,14 @@ static gboolean get_modem_property(struct ofono_modem *modem, const char *name,
 		return FALSE;
 
 	switch (property->type) {
-	case OFONO_PROPERTY_TYPE_STRING:
+	case PROPERTY_TYPE_STRING:
 		*((const char **) value) = property->value;
 		return TRUE;
-	case OFONO_PROPERTY_TYPE_INTEGER:
+	case PROPERTY_TYPE_INTEGER:
 		memcpy(value, property->value, sizeof(int));
 		return TRUE;
-	case OFONO_PROPERTY_TYPE_BOOLEAN:
-		memcpy(value, property->value, sizeof(bool));
+	case PROPERTY_TYPE_BOOLEAN:
+		memcpy(value, property->value, sizeof(ofono_bool_t));
 		return TRUE;
 	default:
 		return FALSE;
@@ -1003,22 +1140,19 @@ static gboolean get_modem_property(struct ofono_modem *modem, const char *name,
 int ofono_modem_set_string(struct ofono_modem *modem,
 				const char *key, const char *value)
 {
-	return set_modem_property(modem, key,
-					OFONO_PROPERTY_TYPE_STRING, value);
+	return set_modem_property(modem, key, PROPERTY_TYPE_STRING, value);
 }
 
 int ofono_modem_set_integer(struct ofono_modem *modem,
 				const char *key, int value)
 {
-	return set_modem_property(modem, key,
-					OFONO_PROPERTY_TYPE_INTEGER, &value);
+	return set_modem_property(modem, key, PROPERTY_TYPE_INTEGER, &value);
 }
 
 int ofono_modem_set_boolean(struct ofono_modem *modem,
-				const char *key, bool value)
+				const char *key, ofono_bool_t value)
 {
-	return set_modem_property(modem, key,
-					OFONO_PROPERTY_TYPE_BOOLEAN, &value);
+	return set_modem_property(modem, key, PROPERTY_TYPE_BOOLEAN, &value);
 }
 
 const char *ofono_modem_get_string(struct ofono_modem *modem, const char *key)
@@ -1026,7 +1160,7 @@ const char *ofono_modem_get_string(struct ofono_modem *modem, const char *key)
 	const char *value;
 
 	if (get_modem_property(modem, key,
-			OFONO_PROPERTY_TYPE_STRING, &value) == FALSE)
+				PROPERTY_TYPE_STRING, &value) == FALSE)
 		return NULL;
 
 	return value;
@@ -1037,18 +1171,18 @@ int ofono_modem_get_integer(struct ofono_modem *modem, const char *key)
 	int value;
 
 	if (get_modem_property(modem, key,
-			OFONO_PROPERTY_TYPE_INTEGER, &value) == FALSE)
+				PROPERTY_TYPE_INTEGER, &value) == FALSE)
 		return 0;
 
 	return value;
 }
 
-bool ofono_modem_get_boolean(struct ofono_modem *modem, const char *key)
+ofono_bool_t ofono_modem_get_boolean(struct ofono_modem *modem, const char *key)
 {
-	bool value;
+	ofono_bool_t value;
 
 	if (get_modem_property(modem, key,
-			OFONO_PROPERTY_TYPE_BOOLEAN, &value) == FALSE)
+				PROPERTY_TYPE_BOOLEAN, &value) == FALSE)
 		return FALSE;
 
 	return value;
@@ -1132,22 +1266,21 @@ static void modem_sim_ready(void *user, enum ofono_sim_state new_state)
 
 	switch (new_state) {
 	case OFONO_SIM_STATE_NOT_PRESENT:
-		if (modem->pre_sim_atoms != NULL)
-			remove_all_atoms(&modem->atoms);
+		modem_change_state(modem, MODEM_STATE_PRE_SIM);
 		break;
 	case OFONO_SIM_STATE_INSERTED:
 		break;
 	case OFONO_SIM_STATE_READY:
-		if (modem->pre_sim_atoms == NULL) {
-			modem->pre_sim_atoms = modem->atoms;
-			modem->atoms = NULL;
-		}
+		modem_change_state(modem, MODEM_STATE_OFFLINE);
 
-		if (modem->driver->post_sim)
-			modem->driver->post_sim(modem);
+		/*
+		 * If we don't have the set_online method, also proceed
+		 * straight to the online state
+		 */
+		if (modem->driver->set_online == NULL)
+			modem_change_state(modem, MODEM_STATE_ONLINE);
 
-		__ofono_history_probe_drivers(modem);
-		__ofono_nettime_probe_drivers(modem);
+		break;
 	}
 }
 
